﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using At.Core.Membership;
using At.Core.TradeFilters;
using At.Infrastructure;
using At.Infrastructure.TradePositionService;
using At.Strategies.Helpers;
using Microsoft.VisualBasic.FileIO;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace AT.RealTests
{
	[TestClass]
	public class OptimizationResultsLoader
	{
		private readonly IDictionary<string, string> _dataFields = new Dictionary<string, string>
		{
			{"Результат из скрипта", "ScriptResult"}
			,{"Чистый П/У", "NetProfit"}
			,{"Чистый П/У %", "NetProfitPct"}
			,{"Общий MFE", "NetMFE"}
			,{"Доходность в год", "CAGR"}
			,{"Доходность в месяц", "CAGRMonth"}
			,{"Доход за бар", "ProfitPerBar"}
			,{"Макс. просадка", "MaxDrawdown"}
			,{"Макс. просадка %", "MaxDrawdownPct"}
			,{"День макс. просадки", "MaxDrawdownDate"}
			,{"День макс. просадки %", "MaxDrawdownPctDate"}
			,{"Количество сделок", "AllTrades"}
			,{"Средний П/У", "AvgProfitLoss"}
			,{"Средний П/У в %", "AvgProfitLossPct"}
			,{"Выиграно сделок в %", "WinTradesPct"}
			,{"Выиграно сделок", "WinTrades"}
			,{"Общая прибыль", "GrossProfit"}
			,{"Макс. подряд прибыльных", "MaxConsectiveWin"}
			,{"Общий убыток", "GrossLoss"}
			,{"Средний убыток", "AvgLoss"}
			,{"Средний убыток в %", "AvgLossPct"}
			,{"Макс. подряд убыточных", "MaxConsectiveLoss"}
			,{"Профит фактор", "ProfitFactor"}
			,{"Фактор восстановления", "RecoveryFactor"}
			,{"Коэф. выигрыша", "PayoffRatio"}
		};

		private readonly IList<ParamInfo> _paramsFields = new List<ParamInfo>
		{
			ParamInfo.Create("ВнешнийСкрипт1:SlowSMAPeriod", "SlowSMAPeriod", ParameterTypes.Int),
			ParamInfo.Create("ВнешнийСкрипт1:FastSMAPeriod", "FastSMAPeriod", ParameterTypes.Int),
			ParamInfo.Create("ВнешнийСкрипт1:ATRPeriod", "ATRPeriod", ParameterTypes.Double),
			ParamInfo.Create("ВнешнийСкрипт1:ATRMultiply", "ATRMultiply", ParameterTypes.Double)
		};

		private readonly IFilter[] _filters =
		{
			new Filter(StopHelper.RiskStopSignalName, FilterType.Exit),
			new Filter(StopHelper.RatchetChannelExitSignalName, FilterType.Exit)
		};

		private List<ColumnInfo> _columnIndexes;
		private FacadeGateway _facadeGateway;
		private string _tradeSystemName;
		private string _tradeSystemVersion;
		private string _securityName;
		private string _exportPath;
		private string _exportFileName;

		private class ColumnInfo
		{
			public static ColumnInfo Column(string name, int index)
			{
				return new ColumnInfo
				{
					Name = name,
					Index = index,
					IsParam = false
				};
			}

			public static ColumnInfo Param(string name, int index)
			{
				return new ColumnInfo
				{
					Name = name,
					Index = index,
					IsParam = true
				};
			}

			public string Name { get; private set; }

			public int Index { get; private set; }

			public bool IsParam { get; private set; }
		}

		private class ParamInfo
		{
			public static ParamInfo Create(string csvName, string mainName, ParameterTypes type)
			{
				return new ParamInfo
				{
					CsvName = csvName,
					MainName = mainName,
					Type = type
				};
			}

			public string CsvName { get; private set; }

			public string MainName { get; private set; }

			public ParameterTypes Type { get; private set; }
		}

		[TestInitialize]
		public void Initialize()
		{
			_facadeGateway = new FacadeGateway();
			_columnIndexes = new List<ColumnInfo>();

			_exportPath = @"C:\Users\menshov\Google Диск\Трейдинг\Результаты тестов";
			_exportFileName = "ReverseBenchmarkEntry_FLTR_SBER_090101_150331_H1.csv";

			_tradeSystemName = TradeSystemNames.ReverseBenchmarkEntryTradeSystem;
			_tradeSystemVersion = TradeSystemVersions.BenchmarkEntryTradeSystem;
			_securityName = "SBER_090101_150331_H1:-";
		}

		[TestMethod]
		public void LoadResults()
		{
			//_facadeGateway.ClearTradeSystemResult(_tradeSystemName, _tradeSystemVersion, _securityName);

			using (var parser = new TextFieldParser(Path.Combine(_exportPath, _exportFileName), Encoding.UTF8))
			{
				parser.TextFieldType = FieldType.Delimited;
				parser.SetDelimiters(";");

				var rowIndex = 0;

				while (!parser.EndOfData)
				{
					var result = new TradeSystemResultDTO();
					var parameters = new List<TradeSystemParameterDTO>();

					string[] fields = parser.ReadFields();

					for (var colIndex = 0; colIndex < fields.Length; colIndex++)
					{
						var field = fields[colIndex];

						if (String.IsNullOrWhiteSpace(field))
							continue;

						if (rowIndex == 0)
							SetColumnIndex(field, colIndex);
						else
						{
							var columnName = GetColumnName(colIndex);

							if (!IsColumnParam(colIndex))
							{
								var propertyInfo = result.GetType().GetProperty(columnName);
								var value = Convert.ChangeType(field, propertyInfo.PropertyType);
								propertyInfo.SetValue(result, value);
							}
							else
							{
								var paramInfo = _paramsFields.First(p => p.MainName == columnName);

								if (paramInfo.Type == ParameterTypes.Int)
									parameters.Add(new TradeSystemIntParameterDTO
									{
										Name = columnName,
										Value = Convert.ToInt32(Convert.ToDouble(field))
									});

								if (paramInfo.Type == ParameterTypes.Double)
									parameters.Add(new TradeSystemDoubleParameterDTO
									{
										Name = columnName,
										Value = Convert.ToDouble(field)
									});
							}
						}
					}

					if (rowIndex > 0)
					{
						var parameterArray = parameters.ToArray();

						if (parameterArray.Any())
							result.Parameters = parameterArray;

						var filtersArray = _filters
							.Select(f => new TradeSystemFilterDTO(f.Name, f.FilterType))
							.ToArray();

						_facadeGateway.AddTradeSystemResult(result, _tradeSystemName, _tradeSystemVersion, _securityName, parameterArray, filtersArray);
					}

					rowIndex++;
				}
			}

		}

		private void SetColumnIndex(string fieldName, int colIndex)
		{
			if (_dataFields.ContainsKey(fieldName))
				_columnIndexes.Add(ColumnInfo.Column(fieldName, colIndex));
			else if (_paramsFields.Any(p => p.CsvName == fieldName))
				_columnIndexes.Add(ColumnInfo.Param(fieldName, colIndex));
			else
				throw new Exception(String.Format("Поле в файле результатов с именем \"{0}\" не найдено ни в коллекции результатов, ни в коллекции параметров.", fieldName));
		}

		private bool IsColumnParam(int colIndex)
		{
			if (_columnIndexes.All(c => c.Index != colIndex))
				throw new Exception(String.Format("Не найдено имя для поля с индексом \"{0}\" ни в коллекции результатов, ни в коллекции параметров.", colIndex));

			var columnInfo = _columnIndexes.First(ci => ci.Index == colIndex);
			return columnInfo.IsParam;
		}

		private string GetColumnName(int colIndex)
		{
			if (_columnIndexes.All(c => c.Index != colIndex))
				throw new Exception(String.Format("Не найдено имя для поля с индексом \"{0}\" ни в коллекции результатов, ни в коллекции параметров.", colIndex));

			var columnInfo = _columnIndexes.First(ci => ci.Index == colIndex);
			var columnName = columnInfo.Name;
			var columnIsParam = columnInfo.IsParam;

			return columnIsParam ? _paramsFields.First(p => p.CsvName == columnName).MainName : _dataFields[columnName];
		}
	}
}